﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ConsoleApplication1
{
    /**
     * Estrutura da ficha
     * 
     */
    public struct Ficha
    {
        public string nome;
        public int idade;
        public double altura;
        public double peso;
    }

    /**
     * 
     * 
     */
    class ControleDeIMC
    {
        // constante contendo o nome do arquivo
        const string ARQUIVO = @"c:\cadastro.txt";

        // número atual de registros
        public static int registros = 0;

        // número máximo de registros
        public static int maxRegistros = 10;

        /**
         * gravarArquivosFichas
         * 
         * Recebe um array de fichas que serão armazenadas
         * no arquivo
         * 
         * Parâmetros:
         * @param Ficha[] pessoas
         * 
         * Retorna:
         * @return nada
         * 
         */
        static public void gravarArquivosFichas(Ficha[] fichas)
        {
            // abre o arquivo para escrita
            FileStream arq = new FileStream(ARQUIVO, FileMode.Create);
            StreamWriter writer = new StreamWriter(arq);

            // escreve o número de fichas cadastrada      
            writer.WriteLine(registros);

            int i = 0;

            // grava todos os registros            
            while (i < registros)
            {
                writer.WriteLine(fichas[i].nome);
                writer.WriteLine(fichas[i].idade);
                writer.WriteLine(fichas[i].peso);
                writer.WriteLine(fichas[i].altura);

                i++;

            }

            // Exibe o número de fichas gravada
            Console.WriteLine("Numero de fichas gravadas: {0}", registros);

            // fecha o arquivo
            writer.Close();

            arq.Close();
           
        }

        /**
         * listarFichas
         * 
         * Lista todas as fichas cadastradas
         *
         * Parâmetros:
         * 
         * Retorna:
         * 
         */
        static public void listarFichas(Ficha[] fichas)
        {
            if ((fichas != null) && (fichas.Length > 0))
            {
                // array contém fichas para serem exibidas
                Console.WriteLine("\nNumero de fichas cadastradas: {0}", registros);

                for (int i = 0; i < registros; i++)
                {
                    // exibe o número do registro
                    Console.WriteLine("Registro: {0}", i);
                    // exibe a ficha atual
                    ExibeFicha(fichas[i]);
                    Console.WriteLine();
                }

                Console.WriteLine();

            }
            else
            {
                // não há registros armazenados
                Console.WriteLine("Lista vazia");
            }
        }

        /**
         * listarFichasIMC
         * 
         * Lista todas as fichas cadastradas
         *
         * Parâmetros:
         * 
         * Retorna:
         * 
         */
        static public void listarFichasIMC(Ficha[] fichas)
        {
            if ((fichas != null) && (fichas.Length > 0))
            {
                Console.WriteLine("\nNumero de fichas cadastradas: {0}", registros);

                for (int i = 0; i < registros; i++)
                {
                    //
                    Console.WriteLine("Registro: {0}", i);
                    ExibeFicha(fichas[i]);
                    Console.WriteLine("IMC: {0:f2}",imc(fichas[i]));
                    Console.WriteLine();
                }

                Console.WriteLine();

            }
            else
            {
                // não há registros
                Console.WriteLine("Lista vazia");
            }
        }


        /**
         * inserirRegistro
         * 
         * Insere uma nova ficha no array
         * 
         * Parâmetros:
         * @param fichas Array onde será armazenada as fichas
         * 
         * Retorna:
         * @return void Nada
         * 
         */
        static public void inserirRegistro(Ficha[] fichas)
        {
            if (registros < maxRegistros)
            {
                // há espaço para nova ficha

                // Obtem informações da pessoa
                Ficha ficha = LerFicha();
                // atualiza o array de fichas e o
                // número atual de fichas
                fichas[registros++] = ficha;

            }
            else
            {
                // não há mais espaços
                Console.WriteLine("Nao ha mais espaco vazio");
            }
        }

        /**
         * lerAquivoFichas
         * 
         * Lê o arquivo e insere todas as fichas em um array
         * 
         * Parâmetros:
         * 
         * Retorna:
         * 
         */
        static public void lerArquivoFichas(ref Ficha[] fichas)
        {
            // lê todos os registros de um arquivo

            // abre o arquivo para leitura
            FileStream arq = new FileStream(ARQUIVO, FileMode.Open);
            StreamReader reader = new StreamReader(arq);

            string linha;

            // lê o número de fichas cadastrada 
            // essa é a primeira informação do arquivo
            int num = int.Parse(reader.ReadLine());

            registros = num;

            if (num > maxRegistros)
            {
                // numero máximo de registros excedido
            }
            else
            {

                Ficha ficha;

                int i = 0;

                // Lê todos os registros
                while ((linha = reader.ReadLine()) != null)
                {
                    ficha.nome = linha;
                    ficha.idade = int.Parse(reader.ReadLine());
                    ficha.peso = double.Parse(reader.ReadLine());
                    ficha.altura = double.Parse(reader.ReadLine());
                    // atualiza o número de fichas
                    fichas[i++] = ficha;
                }
            }

            // Exibe o número de fichas lidas
            Console.WriteLine("Numero de fichas lidas: {0}", registros);

            reader.Close();

            arq.Close();

        }

        /**
         * Exibe uma mensagem e lê uma
         * string do teclado
         * 
         * Parâmetros:
         * @param msg string mensagem exibida
         * 
         * @return string A string lida
         */
        static public String LeString(String msg)
        {
            Console.Write("{0}", msg);
            String s = Console.ReadLine();
            return s;
        }


        /**
         * LerFicha
         * 
         * Lê a ficha de informações de uma pessoa
         * a partir do Console
         * 
         * Parâmetros:
         *   nenhum
         *   
         * Retorna:
         * @return uma ficha
         * 
         * Observe o uso do método LeString(). Ele é
         * chamado várias vezes
         * 
         */
        static public Ficha LerFicha()
        {

            Ficha ficha; // cria uma nova ficha

            ficha.nome = LeString("Entre com o nome:");
            ficha.idade = int.Parse(LeString("Entre com a idade:"));
            ficha.peso = double.Parse(LeString("Entre com o peso (em kg): "));
            ficha.altura = double.Parse(LeString("Entre com a altura (em metros):"));

            // retorna a ficha da pessoa
            return ficha;

        }

        /**
         * imc
         * 
         * Calcula o Indice de Massa Corporea de uma pessoa
         * 
         * imc = (peso)/(altura*altura)
         * 
         * Parâmetros:
         * @param ficha
         * 
         * Retorna:
         * @return double IMC calculado da ficha
         * 
         */
        static public double imc(Ficha ficha)
        {
            return ficha.peso / (Math.Pow(ficha.altura, 2));
        }
        
        /**
         * ExibeFicha
         * 
         * Exibe no console a ficha de um pessoa
         * 
         */
        static public void ExibeFicha(Ficha p)
        {
            Console.WriteLine("Nome  : {0}", p.nome);
            Console.WriteLine("Idade : {0}", p.idade);
            Console.WriteLine("Peso  : {0}", p.peso);
            Console.WriteLine("Altura: {0}", p.altura);
        }

        /**
         * Menu
         * 
         * Exibe e seleciona uma opcao do menu
         * 
         * Retorna a opção selecionada do menu
         * 
         */
        static public int Menu()
        {
            string opcao; // opcão selecionada

            int op=0;

            bool valida = false;

            // repete enquanto a condição não for válida
            do
            {
                
                Console.WriteLine("Menu");
                Console.WriteLine("1-Inserir um registro");
                Console.WriteLine("2-Gravar os registros em um arquivo");
                Console.WriteLine("3-Ler os registros de um arquivo");
                Console.WriteLine("4-Listar todos os registros");
                Console.WriteLine("5-Listar o IMC de todos");
                Console.WriteLine();

                opcao = LeString("Entre com a opcao desejada: ");
                
                if (opcao.Length == 0)
                {
                    opcao = "0";
                }

                op = int.Parse(opcao);

                if ((op < 0) || (op > 5))
                {
                    // opção inválida
                    Console.WriteLine("Opcao {0} e invalida", op);
                    valida = false;
                }
                else
                {
                    // opção válida
                    valida = true;
                }

                Console.WriteLine();

            } while (!valida);

            return op;

        }


        /**
         * Programa principal
         *
         */
        static void Main(string[] args)
        {
          
            Ficha[] fichas = new Ficha[maxRegistros];

            int opcao = 0;

            do
            {

                // exibe e obtem uma opção do Menu
                opcao = Menu();

                switch (opcao)
                {
                    case 0:
                        break;
                    case 1:
                        inserirRegistro(fichas);
                        break;
                    case 2:
                        gravarArquivosFichas(fichas);
                        break;
                    case 3:
                        lerArquivoFichas(ref fichas);
                        break;
                    case 4:
                        listarFichas(fichas);
                        break;
                    case 5:
                        listarFichasIMC(fichas);
                        break;
                    case 6:
                        
                        break;

                }

            } while (opcao != 0);

        }
    }
}
